-- RBT-TEST - МПРД в триггерах на PL/pgSQL
-- Проверка работы МПРД в триггерах на языке PL/pgSQL
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- =========================================================
-- Тестируем триггеры, создаваемые от пользователя postgres
-- =========================================================
--
-- Проверяем работу триггера в таблице без признака MAC CCR
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключается суперпользователь postgres
\connect - postgres
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункUpdate_postgres"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункUpdate';
	RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункUpdate_postgres" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU_postgres"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_postgres"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC_postgres"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункCHMAC';
	RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC_postgres" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBCHMAC_postgres"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункCHMAC_postgres"();
--
-- Просматриваем метку сессии
-- Ожидается {0,0}
SELECT session_maclabel;
-- Просматриваем метки созданных объектов
-- Ожидается метки функций -- {0,0}
-- Метка триггеров -- {2,3}
SELECT maclabel, proname FROM pg_proc WHERE proname = 'ТрФункUpdate_postgres' OR proname = 'ТрФункCHMAC_postgres';
SELECT maclabel, tgname FROM pg_trigger WHERE tgname = 'ТриггерBU_postgres' OR tgname = 'ТриггерBCHMAC_postgres';
--
-- Проверяем модификацию данных в таблице без признака MAC CCR
-- Пользователи меняют данные
-- Ожидается, что метки записей не изменятся
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем таблицу Проверка
DROP TABLE "Проверка" CASCADE;
-- Удаляем триггерные функции
DROP FUNCTION "ТрФункUpdate_postgres"();
DROP FUNCTION "ТрФункCHMAC_postgres"();
--
-- =====================================================
-- Тестируем триггеры, создаваемые от пользователя dbsa
-- =====================================================
--
\connect - dbsa
-- Создаем тестовую таблицу
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем метку сессии
SET ac_session_maclabel = '{1,3}';
-- Просматриваем метку сессии
-- Ожидается {1,3}
SELECT session_maclabel;
--
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
CREATE FUNCTION "ТрФункUpdate_dbsa"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункUpdate';
	RETURN NEW;
END;$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_dbsa"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_dbsa"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
CREATE FUNCTION "ТрФункCHMAC_dbsa"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункCHMAC';
	RETURN NEW;
END;$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBCHMAC_dbsa"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункCHMAC_dbsa"();
--
-- Просматриваем метки созданных объектов
-- Ожидаются метки функций -- {1,3}
-- Метки триггеров -- {2,3}
SELECT maclabel, proname FROM pg_proc WHERE proname = 'ТрФункUpdate_dbsa' OR proname = 'ТрФункCHMAC_dbsa';
SELECT maclabel, tgname FROM pg_trigger WHERE tgname = 'ТриггерBU_dbsa' OR tgname = 'ТриггерBCHMAC_dbsa';
--
-- Назначаем права на триггерные функции
\connect - postgres
GRANT EXECUTE ON FUNCTION "ТрФункUpdate_dbsa" () TO PUBLIC;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC_dbsa" () TO PUBLIC;
--
-- Пытаемся обновить записи таблицы
-- Ожидается сообщение с cache lookup failed для всех
-- пользователей, метка которых меньше {1,3} -- т.к. триггерная функция
-- имеет метку {1,3}
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_0_00
-- =====================================================
--
\connect - u_0_00
-- Просматриваем метку пользователя
-- Ожидается {0,0}
SELECT session_maclabel;
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
-- Ожидается успех
CREATE FUNCTION "ТрФункUpdate_u00"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункUpdate';
	RETURN NEW;
END;$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается ошибка доступа,
-- т.к. метка пользователя не совпадает с меткой таблицы Проверка
CREATE TRIGGER "ТриггерBU_u00"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_u00"();
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_2_11
-- =====================================================
--
\connect - u_2_11
-- Просматриваем метку пользователя
-- Ожидается {2,3}
SELECT session_maclabel;
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
-- Ожидается успех
CREATE FUNCTION "ТрФункUpdate_u23"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
	NEW."данные" = NEW."данные" || '-ТрФункUpdate';
	RETURN NEW;
END;$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается успех
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_u23"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_u23"();
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'