-- RBT-TEST - МПРД в триггерах на PL/Pythonu
-- Проверка работы МПРД в триггерах на языке PL/Pythonu
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-plpythonu.sql'
--
-- =========================================================
-- Тестируем триггеры, создаваемые от пользователя postgres
-- =========================================================
--
-- Проверяем работу триггера в таблице без признака MAC CCR
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключается суперпользователь postgres
\connect - postgres
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION proc_update_postgres() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункUpdate"
	return "MODIFY"
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION proc_update_postgres() TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU_postgres"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_update_postgres();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION proc_chmac_postgres() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункCHMAC"
	return "MODIFY"
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION proc_chmac_postgres() TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBCHMAC_postgres"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_chmac_postgres();
--
-- Просматриваем метку сессии
-- Ожидается {0,0}
SELECT session_maclabel;
-- Просматриваем метки созданных объектов
-- Ожидается метки функций -- {2,3}
-- Метка триггеров -- {0,0}
SELECT maclabel, proname FROM pg_proc WHERE proname = 'proc_update_postgres' OR proname = 'proc_chmac_postgres';
SELECT maclabel, tgname FROM pg_trigger WHERE tgname = 'ТриггерBU_postgres' OR tgname = 'ТриггерBCHMAC_postgres';
--
-- Проверяем модификацию данных в таблице без признака MAC CCR
-- Пользователи меняют данные
-- Ожидается, что метки записей не изменятся
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем таблицу Проверка
DROP TABLE "Проверка" CASCADE;
-- Удаляем триггерные функции
DROP FUNCTION proc_update_postgres();
DROP FUNCTION proc_chmac_postgres();
--
-- =====================================================
-- Тестируем триггеры, создаваемые от пользователя dbsa
-- =====================================================
--
\connect - dbsa
-- Создаем тестовую таблицу
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем метку сессии
SET ac_session_maclabel = '{1,3}';
-- Просматриваем метку сессии
-- Ожидается {1,3}
SELECT session_maclabel;
--
--
\connect - postgres
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
CREATE FUNCTION proc_update_dbsa() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункUpdate"
	return "MODIFY"
$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_dbsa"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_update_dbsa();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
CREATE FUNCTION proc_chmac_dbsa() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункCHMAC"
	return "MODIFY"
$$ SECURITY DEFINER;
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBCHMAC_dbsa"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_chmac_dbsa();
--
-- Меняем владельца объектов на dbsa
ALTER FUNCTION proc_update_dbsa() OWNER TO dbsa;
ALTER FUNCTION proc_chmac_dbsa() OWNER TO dbsa;
-- Изменяем метку объектов на {1,3}
MAC LABEL ON FUNCTION proc_update_dbsa() IS '{1,3}';
MAC LABEL ON FUNCTION proc_chmac_dbsa() IS '{1,3}';
--
\connect - dbsa
--
-- Просматриваем метки созданных объектов
-- Ожидаются метки функций -- {1,3}
-- Метки триггеров -- {2,3}
SELECT maclabel, proname FROM pg_proc WHERE proname = 'proc_update_dbsa' OR proname = 'proc_chmac_dbsa';
SELECT maclabel, tgname FROM pg_trigger WHERE tgname = 'ТриггерBU_dbsa' OR tgname = 'ТриггерBCHMAC_dbsa';
--
-- Назначаем права на триггерные функции
\connect - postgres
GRANT EXECUTE ON FUNCTION proc_update_dbsa() TO PUBLIC;
GRANT EXECUTE ON FUNCTION proc_chmac_dbsa() TO PUBLIC;
--
-- Пытаемся обновить записи таблицы
-- Ожидается сообщение с cache lookup failed для всех
-- пользователей, метка которых меньше {1,3} -- т.к. триггерная функция
-- имеет метку {1,3}
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Cоздаем триггерные функции для пользователей u_0_00
CREATE FUNCTION proc_update_u00() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункUpdate"
	return "MODIFY"
$$ SECURITY DEFINER;
--
CREATE FUNCTION proc_update_u23() RETURNS trigger LANGUAGE plpythonu AS $$
	TD["new"]["данные"] = TD["new"]["данные"] + "-ТрФункUpdate"
	return "MODIFY"
$$ SECURITY DEFINER;
--
-- Изменяем владельцев функций и мандатные атрибуты
ALTER FUNCTION proc_update_u00() OWNER TO u_0_00;
ALTER FUNCTION proc_update_u23() OWNER TO u_2_11;
MAC LABEL ON FUNCTION proc_update_u23() IS '{2,3}';
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_0_00
-- =====================================================
--
\connect - u_0_00
-- Просматриваем метку пользователя
-- Ожидается {0,0}
SELECT session_maclabel;
--
-- Создаем триггер
-- Ожидается ошибка доступа,
-- т.к. метка пользователя не совпадает с меткой таблицы Проверка
CREATE TRIGGER "ТриггерBU_u00"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_update_u00();
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_2_11
-- =====================================================
--
\connect - u_2_11
-- Просматриваем метку пользователя
-- Ожидается {2,3}
SELECT session_maclabel;
--
-- Создаем триггер
-- Ожидается успех
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_u23"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE proc_update_u23();
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
